'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.connect = undefined;

var _mongodb = require('mongodb');

var _Item = require('./lib/Item');

var _User = require('./lib/User');

var _UserItem = require('./lib/UserItem');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const debug = (0, _debug2.default)('data-access:main');

const {
  DB_DATABASE: dbName,
  DB_HOST: host,
  DB_PORT: port,
  DB_POOL_SIZE: poolSize
} = process.env;

const url = `mongodb://${host}:${port}`;

let client;

const connect = exports.connect = (() => {
  var _ref = _asyncToGenerator(function* () {
    try {
      if (!client) {
        debug(`Connecting to ${url}`);
        client = yield _mongodb.MongoClient.connect(url, { poolSize });
        debug('Connected');
      }
    } catch (e) {
      debug(e);
      throw e;
    }

    const db = client.db(dbName);

    return {
      Item: {
        save: (0, _Item.save)(db),
        get: (0, _Item.get)(db),
        list: (0, _Item.list)(db),
        check: _Item.check
      },
      User: {
        save: (0, _User.save)(db),
        get: (0, _User.get)(db),
        list: (0, _User.list)(db)
      },
      UserItem: {
        add: (0, _UserItem.add)(db),
        get: (0, _UserItem.get)(db),
        update: (0, _UserItem.update)(db),
        list: (0, _UserItem.list)(db),
        remove: (0, _UserItem.remove)(db)
      }
    };
  });

  return function connect() {
    return _ref.apply(this, arguments);
  };
})();