'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.remove = exports.list = exports.get = exports.update = exports.add = undefined;

var _User = require('./User');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const debug = (0, _debug2.default)('data-access:userItem');

const STORES = ['amazon', 'bestbuy', 'walmart', 'target', 'gamestop'];

const buildUserItem = db => (() => {
  var _ref = _asyncToGenerator(function* (userItemDoc) {
    const { userId, goal: price, updatedAt, result, groupId } = userItemDoc;
    const items = yield Promise.all(STORES.map(function (store) {
      return db.collection(store).findOne({ groupId });
    })).then(function (s) {
      return s.filter(function (_s) {
        return !!_s;
      });
    });

    if (!items || items.length === 0) {
      throw new Error(`No Items Found with groupId: ${groupId}`);
    }

    // Get Global Value
    const [{ title, msrp }] = items;
    const goal = { price, result };
    const offers = items.map(function (i) {
      const { url, itemId, store, updatedAt, offers } = i;
      const offer = offers.sort(function (x, y) {
        return x.price - y.price;
      }).pop();
      return Object.assign({}, offer, { url, itemId, store, updatedAt });
    });

    // Get lowest Priced Offer
    const offer = offers.sort(function (x, y) {
      return x.price - y.price;
    }).pop();

    return {
      goal,
      userId,
      groupId,
      title,
      msrp,
      updatedAt,
      offer,
      offers
    };
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

const add = exports.add = db => (() => {
  var _ref2 = _asyncToGenerator(function* (input) {
    const { userId, groupId } = input;
    const now = new Date();

    try {
      const user = yield (0, _User.get)(db)(userId);

      if (!user) {
        throw new Error('Invalid User');
      }

      const { value } = yield db.collection('userItems').findOneAndUpdate({ userId, groupId }, {
        $set: Object.assign({}, input, { updatedAt: now }),
        $setOnInsert: { createdAt: new Date(), result: false }
      }, { returnOriginal: false, upsert: true });

      return buildUserItem(db)(value);
    } catch (e) {
      debug(e);
      throw e;
    }
  });

  return function (_x2) {
    return _ref2.apply(this, arguments);
  };
})();

const update = exports.update = db => (() => {
  var _ref3 = _asyncToGenerator(function* (input) {
    const { userId, groupId, goal } = input;

    try {
      const { value } = yield db.collection('userItems').findOneAndUpdate({ userId, groupId }, {
        $set: { goal }
      }, { returnOriginal: false });

      return buildUserItem(db)(value);
    } catch (e) {
      throw e;
    }
  });

  return function (_x3) {
    return _ref3.apply(this, arguments);
  };
})();

const get = exports.get = db => (() => {
  var _ref4 = _asyncToGenerator(function* (query) {
    const { userId, groupId } = query;

    try {
      const _userItem = yield db.collection('userItems').findOne({ userId, groupId }, { _id: 0 });

      return buildUserItem(db)(_userItem);
    } catch (e) {
      debug(e);
      throw e;
    }
  });

  return function (_x4) {
    return _ref4.apply(this, arguments);
  };
})();

const list = exports.list = db => (() => {
  var _ref5 = _asyncToGenerator(function* (userId) {
    try {
      const userItemDocs = yield db.collection('userItems').find({ userId }).toArray();

      return Promise.all(userItemDocs.map(buildUserItem(db)));
    } catch (e) {
      debug(e);
      throw e;
    }
  });

  return function (_x5) {
    return _ref5.apply(this, arguments);
  };
})();

const remove = exports.remove = db => (() => {
  var _ref6 = _asyncToGenerator(function* ({ userId, groupId }) {
    try {
      const { result } = yield db.collection('userItems').remove({ userId, groupId });
      if (result.n === 0) {
        throw new Error('Item Removal Error');
      }
      return result.n === 1;
    } catch (e) {
      debug(e);
      throw e;
    }
  });

  return function (_x6) {
    return _ref6.apply(this, arguments);
  };
})();