'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
const Sequelize = require('sequelize');
const uuid = require('uuid/v4');

const {
  DB_USERNAME: username,
  DB_PASSWORD: password,
  DB_DATABASE: database,
  DB_HOST: host
} = process.env;

console.log(`Connecting to ${host}/${database} using ${username}`);

const sequelize = exports.sequelize = new Sequelize(database, username, password, {
  host,
  dialect: 'postgres',
  operatorsAliases: false,

  pool: {
    max: 5,
    min: 0,
    acquire: 30000,
    idle: 10000
  }
});

const Item = exports.Item = sequelize.define('item', {
  id: {
    type: Sequelize.UUID,
    primaryKey: true,
    unique: true,
    defaultValue: () => uuid()
  },
  itemId: {
    type: Sequelize.TEXT,
    unique: 'compositeIndex',
    allowNull: false
  },
  store: {
    type: Sequelize.ENUM('amazon', 'bestbuy', 'gamestop', 'target'),
    unique: 'compositeIndex',
    allowNull: false
  },
  upc: Sequelize.TEXT,
  lgImage: Sequelize.TEXT,
  manufacturer: Sequelize.TEXT,
  mdImage: Sequelize.TEXT,
  msrp: Sequelize.INTEGER,
  offers: Sequelize.ARRAY(Sequelize.JSONB),
  offer: Sequelize.JSONB,
  title: Sequelize.TEXT,
  ttl: Sequelize.INTEGER,
  updatedAt: Sequelize.DATE,
  url: Sequelize.TEXT,
  valid: { type: Sequelize.BOOLEAN, allowNull: false, defaultValue: true }
}, {
  indexes: [{
    unique: true,
    fields: ['itemId', 'store']
  }]
});

const UserItem = exports.UserItem = sequelize.define('userItem', {
  id: {
    type: Sequelize.UUID,
    allowNull: false,
    primaryKey: true,
    defaultValue: () => uuid(),
    unique: true
  },
  userId: {
    type: Sequelize.TEXT,
    allowNull: false
  },
  results: {
    type: Sequelize.BOOLEAN,
    defaultValue: false
  },
  goal: {
    type: Sequelize.INTEGER,
    allowNull: true
  },
  itemId: {
    type: Sequelize.UUID,
    allowNull: false
  },
  interval: {
    type: Sequelize.INTEGER,
    defaultValue: 1
  }
});

const User = exports.User = sequelize.define('user', {
  userId: {
    type: Sequelize.TEXT,
    allowNull: false,
    primaryKey: true,
    unique: true
  },
  email: Sequelize.ARRAY(Sequelize.TEXT),
  phone: Sequelize.INTEGER,
  token: Sequelize.TEXT,
  carrier: Sequelize.TEXT
});

UserItem.belongsTo(Item, { foreignKey: 'itemId', targetKey: 'id' });

Item.sync().then(() => {
  User.sync().then(() => {
    UserItem.sync();
  });
});