// @flow
declare type SupportedStores = 'amazon' | 'bestbuy' | 'walmart' | 'target' | 'gamestop'
declare type Reason = 'NEW ITEM' | 'AVAILABLE' | 'LOW PRICE'

declare type Category = {
  id: string,
  name: string
}

declare type ShippingInfo = {
  price: ?string
}

declare type Offer = {
  condition: ?string,
  seller: ?string,
  price: number,
  available: boolean,
  shippingInfo: ShippingInfo,
  marketplace: boolean
}

type History = { date: Date }
type OfferHistory = Offer & History

declare type RakutenItem = {
  itemId: string,
  upc: ?string,
  store: SupportedStores,
  title: string,
  manufacturer: ?string,
  url: string,
  msrp: ?number,
  offers: Array<Offer>,
  thumbImage: ?string,
  mdImage: ?string,
  lgImage: ?string,
  category: Category
}

declare type CheckResult = {
  store: SupportedStores,
  itemId: string,
  reason: Reason
}

declare type CheckCallback = (CheckResult) => void

declare type Item = {
  _id: string,
  itemId: string,
  groupId: string,
  upc: ?string,
  store: SupportedStores,
  title: string,
  manufacturer: ?string,
  url: string,
  msrp: ?number,
  offers: Array<Offer>,
  offerHistory: Array<OfferHistory>,
  thumbImage: ?string,
  mdImage: ?string,
  lgImage: ?string,
  category: ?Category,
  updatedAt: Date,
  createdAt: Date
}

declare type ItemQuery = {
  _id?: string,
  itemId?: string,
  upc?: string,
  groupId?: string,
  category?: Category
}

declare type User = {
  _id: string,
  id: string,
  token: string
}

declare type UserInput = {
  id: string,
  token: string
}

declare type UserItemOffer = Offer & {
  url: string,
  itemId: string,
  store: SupportedStores,
  updatedAt: Date
}

declare type Goal = {
  price: number,
  result: boolean
}

declare type UserItemInput = {
  userId: string,
  groupId: string,
  goal: number
}

declare type UserItemDoc = {
  _id: string,
  userId: string,
  groupId: string,
  goal: number,
  result: boolean,
  createdAt: Date,
  updatedAt: Date
}

declare type UserItem = {
  userId: string,
  title: string,
  msrp: ?number,
  groupId: string,
  goal: Goal,
  updatedAt: Date,
  offer: UserItemOffer,
  offers: Array<UserItemOffer>
}

declare type UserItemQuery = {
  groupId: string,
  userId: string,
  store: SupportedStores
}
