// @flow
import _ from 'lodash'
const moment = require('moment')
import * as CONSTANT from './constants.js'
const stopword = require('stopword')

export const getAdded = (fbDB) => async (store) => {
  if(store === null || store === undefined){
    const items: Array<Item> = await Promise.all(CONSTANT.STORES.map(async (store) => {
      let list = await getPathValue(fbDB, `${store}/recentAdded`)
      return Promise.resolve(list === null ? []: list)
    }))
    //console.log()
    let itemList = await Promise.all(_.flatMap(items).map(async (path) => {
      let item = await getPathValue(fbDB, `${path}`)
      return Promise.resolve(item)
    }))

    return Promise.resolve(itemList.sort((x,y) => new Date(y.createdAt) - new Date(x.createdAt)))
  }

  return []
}



export const getUpdated = (fbDB) => async (store) => {
  if(store === null || store === undefined){
    const items: Array<Item> = await Promise.all(CONSTANT.STORES.map(async (store) => {
      let list = await getPathValue(fbDB, `${store}/recentUpdated`)
      return Promise.resolve(list === null ? []: list)
    }))

    let itemList = await Promise.all(_.flatMap(items).map(async (path) => {
      let item = await getPathValue(fbDB, `${path}`)
      return Promise.resolve(item)
    }))

    return Promise.resolve(itemList.sort((x,y) => new Date(y.updatedAt) - new Date(x.updatedAt)))
  }

  return []
}

export const purge =  (item) => {
    let purge = item.offerHistory.reduce((total, offer) => {
      if(!Array.isArray(total)){
        total = [total]
      }

      let previous = total[total.length-1]
      const previousDate = new Date(previous.date)
      const offerDate = new Date(offer.date)
      let previousYear = previousDate.getFullYear()
      let previousMonth = previousDate.getMonth()
      let offerYear = offerDate.getFullYear()
      let offerMonth = offerDate.getMonth()
      if(offerYear === previousYear && offerMonth === previousMonth) {//same month same year
        //find lowest price and replace
        total[total.length-1] = previous.price < offer.price ? previous : offer
        return total
      } else {//if offer is in currentMonth and Year, just append
        return total.concat(offer)
      }
    })

    return purge
}

export const getViewAll = (fbDB) => async (type) => {
  if(type === 'price'){
    return Promise.resolve(await getUpdated(fbDB)())
  } else if (type === 'new'){
    return Promise.resolve(await getAdded(fbDB)())
  }

  return []
}

export const getByGroupID = (fbDB) => async (id) => {
  let pathList = []
  await Promise.all(CONSTANT.STORES.map(async (store) => {
    let storeObj = await getPathValue(fbDB, `${store}/upc`)

    if(storeObj !== null && storeObj !== undefined){
      for(var key in storeObj) {
        if(key.includes(id)){
          pathList = pathList.concat(storeObj[key])
        }
      }
    }

    return Promise.resolve()
  }))

  const items = await Promise.all(pathList.map(async (path) => {
    let item = await getPathValue(fbDB, `${path}`)
    return Promise.resolve(item)
  }))

  return Promise.resolve(items.sort((x,y) => x.price - y.price))
}

const getPathValue =  async (fbDB, path) => {
  let snapshot = await fbDB.ref(`${path}`).once('value')
  return snapshot.val()
}

export const getByName = (fbDB) => async (title, platform: Platform, searchType: SearchType) => {
  let result = []

  if(title && platform === undefined && searchType === undefined){//search only by name
    title = wash(title.toLowerCase())
    let mergeList = []
    await Promise.all(title.map(async (word) => {
      mergeList = await getPathValue(fbDB, `meta/${word}`)
      return Promise.resolve()
    }))

      const items = await Promise.all(_.uniq(mergeList).map(async (path) => {
        let item = await getPathValue(fbDB, `${path}`)
        return Promise.resolve(item)
      }))

      //let temp = items.filter(obj => obj.title.toLowerCase().includes(title))
      result.push(items)
  } else {
    //search by title category and platform
    let mergeList = []
    if(title.includes('[0-9]'))
    {
      title = '0'
    }
    title = title.slice(0,1).toUpperCase()
    if(!isNaN(parseInt(title))){//check to see if game starts with a number
      title = '0-9'
    }
    let categoryList = []
    await Promise.all(CONSTANT.STORES.map(async (store) => {
      let list = await getPathValue(fbDB, `${store}/category/${title}`)

      if(list !== null){
        categoryList = categoryList.concat(list)
      }

      return Promise.resolve()

    }))

    let platformList = []
    await Promise.all(CONSTANT.STORES.map(async (store) => {
      let list = await getPathValue(fbDB, `${store}/platform/${platform}`)

      if(list !== null){
        platformList = platformList.concat(list)
      }

      return Promise.resolve()

    }))

    //merge intersection paths
    mergeList = _.intersection(categoryList, platformList)
    const items = await Promise.all(mergeList.map(async (path) => {
      let item = await getPathValue(fbDB, `${path}`)
      return Promise.resolve(item)
    }))

    result.push(items)
  }

  return Promise.resolve(_.flatMap(result).sort((x,y) => moment(y.updatedAt).isAfter(moment(x.updatedAt))))
}

// export const fixPlatform = async (store, upc, newPlatform) => {
//   //using store/upc/${upc} to get item path (walmart, 886162560428) => walmart/55488478 and item
//   let itemPath = await fbDB.ref(`${store}/upc/${upc}`).once('value').then( function(snapshot) {
//     return snapshot.val()
//   })
//   let item = await fbDB.ref(`${itemPath}`).once('value').then( function(snapshot) {
//     return snapshot.val()
//   })
//   let {platform} = item//get old platform from item
//
//   //update platform on item using the new path and NEW platform fields
//   let itemPlatformMap = {}
//   itemPlatformMap[`${itemPath}/platform`] = `${newPlatform}`
//   await fbDB.ref().update(itemPlatformMap)
//   //remove id(55488478) from store/platform dir
//   let newPlatformList = await updateItemsFromPath(`${store}/platform/${platform}`, [`${itemPath}`])//delete old platform
//   let removeMap = {}
//   removeMap[`${store}/platform/${platform}`] = newPlatformList
//   await fbDB.ref().update(removeMap)//updating the old platform list
//   //add id(55488478) to store/NEW platform
//   let newList = await fbDB.ref(`${store}/platform/${newPlatform}`).once('value').then( function(snapshot) {
//       let oldValueList = snapshot.val()
//       if(oldValueList !== null && oldValueList !== undefined){
//         return Promise.resolve(oldValueList.concat(`${itemPath}`))
//       }
//       return Promise.resolve(oldValueList)
//     })
//
//     let storePlatformMap = {}
//     storePlatformMap[`${store}/platform/${newPlatform}`] = _.uniq(newList)
//     return fbDB.ref().update(storePlatformMap)//updating the new platform list
// }

export const updateItemsFromPath = async (path, value) => {
  let newList = await fbDB.ref(path).once('value').then( function(snapshot) {
    let oldList = snapshot.val()
    oldList = _.difference(oldList, value)
    return Promise.resolve(oldList)
  })
  return Promise.resolve(newList)
}

export const save = (fbDB) => async (store, itemMap, dataMap, removeMap) => {
  //update item first
  await fbDB.ref().update(itemMap)
  //remove shared data, then add share data
  for(var key in removeMap) {
    let newList = await updateItemsFromPath(key, removeMap[key])
    removeMap[key] = newList
  }

await fbDB.ref().update(removeMap)
//check to see if something is being added
let ralist = dataMap[`${store}/recentAdded`]
if(ralist !== null && ralist !== undefined){
  await fbDB.ref(`${store}/recentAdded`).once('value').then( function(snapshot) {
    dataMap[`${store}/recentAdded`] = _.uniq(ralist.concat(snapshot.val())).slice(0, 12)
  })
}

//check to see if something is being updated
let rulist = dataMap[`${store}/recentUpdated`]
if(rulist !== null && rulist !== undefined){
  await fbDB.ref(`${store}/recentUpdated`).once('value').then( function(snapshot) {
    dataMap[`${store}/recentUpdated`] =  _.uniq(rulist.concat(snapshot.val())).slice(0, 12)
  })
}

//check platform, category and priceRange
for(var key in dataMap) {
  if(key.includes('category')){
    await fbDB.ref(key).once('value').then( function(snapshot) {
        let oldValueList = snapshot.val()
        if(oldValueList !== null && oldValueList !== undefined){
          dataMap[key] = oldValueList.concat(dataMap[key])
        }
      })
  }

  if(key.includes('platform')){
    await fbDB.ref(key).once('value').then( function(snapshot) {
        let oldValueList = snapshot.val()
        if(oldValueList !== null && oldValueList !== undefined){
          dataMap[key] = oldValueList.concat(dataMap[key])
        }
      })
  }

  if(key.includes('priceRange')){
    await fbDB.ref(key).once('value').then( function(snapshot) {
        let oldValueList = snapshot.val()
        if(oldValueList !== null && oldValueList !== undefined){
          dataMap[key] = oldValueList.concat(dataMap[key])
        }
      })
  }

  if(key.includes('upc')){
    await fbDB.ref(key).once('value').then( function(snapshot) {
        let oldValueList = snapshot.val()
        if(oldValueList !== null && oldValueList !== undefined){
          dataMap[key] = oldValueList.concat(dataMap[key])
        }
      })
  }

  if(key.includes('meta')){
    await fbDB.ref(key).once('value').then( function(snapshot) {
        let oldValueList = snapshot.val()
        if(oldValueList !== null && oldValueList !== undefined){
          //START DELETE THIS uniq method only logic after once we go live
          dataMap[key] = _.uniq(oldValueList.concat(dataMap[key]))
        }
      })
  }
}

//console.log('updateToFirebase---------------------------------------------')
// console.log(map[`${store}/recentUpdated`])
// console.log(map[`${store}/recentAdded`])
  await fbDB.ref().update(dataMap)
//console.log('FINISH updateToFirebase---------------------------------------------')
  return Promise.resolve()
}

const wash = (text) => {
  let result = stopword.removeStopwords(text.trim().replace( /[^A-Za-z0-9 ]/ig, '').toLowerCase().split(' '))
  return result.filter(x => x.length > 0)
}


export const refreshUpdate = (fbDB) => () => {
  let map = {}
  map['updatedAt'] = new Date().toLocaleString("en-US", {timeZone: "America/New_York"})
  fbDB.ref().update(map)
}


export const parseData = (fbDB) => async (item) => {
  //let newPath = item.upc === null ? `${item.store}/${item.id}` : `${item.store}/${item.upc}/${item.id}`
  let newPath = `${item.store}/${item.id}`
  let updateData = {}
  let removeData = {}

  let oldValue = await fbDB.ref(newPath).once('value').then( function(snapshot) {
    return snapshot.val()
})


return new Promise(async resolve => {
  let result = check(item, oldValue)// returns {store, value, reason}

  if(result.reason.includes(CONSTANT.NEW_ITEM)) {
    await fbDB.ref(newPath).set(result.value, async function(error) {
        if (error) {
          console.log('failed insert')
          console.log(error)
        } else {
          //console.log(`Saving ${NEW_ITEM} to DB`)
          //console.log(result.value.title)
          if(result.value.meta.length > 0){
            result.value.meta.forEach((meta, index) => updateData[`meta/${meta}`] = [newPath])
            //updateItem[`${newPath}/meta`] = result.value.meta
          }


          let category = result.value.meta.join(' ').slice(0,1).toUpperCase()
          if(!isNaN(parseInt(category))){//check to see if game starts with a number
            category = '0-9'
          }

          let priceRange = result.value.offer.priceRange
          let platform = result.value.platform
          updateData[`${result.value.store}/category/${category}`] = [newPath]
          updateData[`${result.value.store}/priceRange/${priceRange}`] = [newPath]
          updateData[`${result.value.store}/platform/${platform}`] = [newPath]
          updateData[`${result.value.store}/recentAdded`] = [newPath]
          updateData[`${result.value.store}/recentUpdated`] = [newPath]
          updateData[`${result.value.store}/upc/${result.value.groupId}`] = [newPath]
        }

        return Promise.resolve()
      })

      return resolve({ updateData,  result, removeData})


  } else if( result.reason.includes(CONSTANT.AVAILABILITY_CHANGE) || result.reason.includes(CONSTANT.MINOR_CHANGE) ||
    result.reason.includes(CONSTANT.AVAILABLE)){
      let updateItem = {}

      //START DELETE THIS url meta upc logic after once we go live
      updateItem[`${newPath}/url`] = result.value.url
      updateItem[`${newPath}/description`] = result.value.description
      updateItem[`${newPath}/groupId`] = result.value.groupId
      updateData[`${result.value.store}/upc/${result.value.groupId}`] = [newPath]
      removeData[`${result.value.store}/upc/${oldValue.groupId}`] = null
      if(result.value.meta.length > 0){
        result.value.meta.forEach((meta, index) => updateData[`meta/${meta}`] = [newPath])
        updateItem[`${newPath}/meta`] = result.value.meta
      }
      //END DELETE THIS

      //update offer for all scenarios
      updateItem[`${newPath}/offer`] = result.value.offer
      updateItem[`${newPath}/offers`] = result.value.offers

      if (result.reason.includes(CONSTANT.MINOR_CHANGE)){
          //only thumbImage, or anything that has to due with the Item specific
          updateItem[`${newPath}/lgImage`] = result.value.lgImage
          updateItem[`${newPath}/title`] = result.value.title
          updateItem[`${newPath}/url`] = result.value.url
      }

      if(result.reason.includes(CONSTANT.AVAILABLE)){
        //available, update current offerings so data doesnt stay stale
        updateItem[`${newPath}/updatedAt`] = result.value.updatedAt
        updateItem[`${newPath}/offerHistory`] = result.value.offerHistory

        if(result.reason.includes(CONSTANT.AVAILABILITY_CHANGE) || (result.reason.includes(CONSTANT.PRICE_CHANGE) && result.reason.includes(CONSTANT.PRICE_LOW))){
          updateData[`${result.value.store}/recentUpdated`] = [newPath]
        }

        if(result.reason.includes(CONSTANT.PRICE_CHANGE)){
          //only offer and offers values should be updated here
          let priceRange = result.value.offer.priceRange
          updateData[`${result.value.store}/priceRange/${priceRange}`] = [newPath]

        //remove old priceRange
        if(result.reason.includes(CONSTANT.NEW_OFFER)){
          let oldPriceRange = result.value.offerHistory[result.value.offerHistory.length - 2].priceRange //2 because we already added a new history in the list
          removeData[`${result.value.store}/priceRange/${oldPriceRange}`] = [newPath]
          result.value.offerHistory = purge(result.value)//purge only when NEW OFFER gets added to history and after we find the priceRange of the last NEW OFFER
          updateItem[`${newPath}/offerHistory`] = result.value.offerHistory//update history after purge
        }
        }
      }//end else if


      return resolve({updateItem, updateData,  result, removeData})
  }//end else if
return resolve(null)

})//end promise
}

const constructNewValue =  (newValue, lowestNewOffer, offerHistory) => {
  let metaList = wash(newValue.title)
  return Object.assign({}, newValue, {meta: metaList, hot: false, offer: lowestNewOffer, offerHistory, updatedAt: Date.now(), createdAt: Date.now(), groupId: newValue.upc || newValue.id })
}

const constructUpdateValue = (newValue, hotItem,lowestNewOffer, offerHistory) => {
    let metaList = wash(newValue.title)// DELETE THIS patch url meta logic after once we go live
    return Object.assign({}, newValue, {meta: metaList, hot: hotItem, url: newValue.url, thumbImage: newValue.thumbImage,mdImage: newValue.mdImage, lgImage:  newValue.lgImage, title: newValue.title , offer: lowestNewOffer, offerHistory, updatedAt: Date.now(), groupId: newValue.upc || newValue.id })
}

const testPriceChange = (reason, lowestNewOffer, oldValue) => {
  //test to see if price changed
  if (lowestNewOffer.price < oldValue.msrp && lowestNewOffer.price < oldValue.offer.price) {//check how much percentage off msrp
    reason.push(CONSTANT.PRICE_CHANGE,CONSTANT.PRICE_LOW)
    let percentageOff = lowestNewOffer.price/oldValue.msrp

    if(percentageOff <= .5){// percentage off
      reason.push(PRICE_50Off)
    } else if(percentageOff <= .8){
      reason.push(PRICE_20Off)
    } else if(percentageOff <= .9){
      reason.push(PRICE_10Off)
    }

    if (lowestNewOffer.price < 20){//price is below $20
      reason.push(CONSTANT.PRICE_BELOW_20)
    }
  }  else if(lowestNewOffer.price > oldValue.msrp || lowestNewOffer.price > oldValue.offer.price) {
    reason.push(CONSTANT.PRICE_CHANGE, CONSTANT.PRICE_HIGH)
  }
  return reason
}

const testMinorChange = (oldValue, newValue) => {
  if(((oldValue.lgImage === undefined  || oldValue.lgImage === null ||
     oldValue.lgImage.search("no_image") >= 0)
    && (newValue.lgImage !== undefined && newValue.lgImage !== null && newValue.lgImage.search("no_image") < 0 )) ||
    (oldValue.url !== newValue.url) ||
    (oldValue.title !== newValue.title)
  ){
    return [CONSTANT.MINOR_CHANGE]
  }

  return []
}

const check =  (newValue: any, oldValue: any) => {
  const { store, id } = newValue
  let newAvailOff = newValue.offers.filter(offer => offer.available === true)// find only avail items
  if (oldValue === null) {// item never existed so we need to add new item
    if(newAvailOff.length > 0) {  // find lowestNewOffer from one the avail new offer
      const [lowestNewOffer] = newAvailOff.sort((x, y) => x.price - y.price)
      return { store, value:  constructNewValue(newValue, lowestNewOffer, [lowestNewOffer]) , reason: [CONSTANT.NEW_ITEM, AVAILABLE, CONSTANT.AVAILABILITY_CHANGE] }
    } else { //nothing is avail, just find lowestNewOffer
      return { store, value:  constructNewValue(newValue, newValue.offer, newValue.offers) , reason: [CONSTANT.NEW_ITEM]}
    }
  } else if (oldValue != null) {// item already exist in DB
    // find lowestNewOffer from one the avail new offer
    const [lowestNewOffer] = newAvailOff.sort((x, y) => x.price - y.price)
    const hotItem = oldValue.hot !== undefined ? oldValue.hot : false
    if (!oldValue.offer.available && newAvailOff.length > 0) {//means item went from NOT avail to avail
      let reason = [CONSTANT.AVAILABLE, CONSTANT.AVAILABILITY_CHANGE]
      // only add to offerHistory if the price changes and item is available
      if(oldValue.offer.price !== lowestNewOffer.price){
        if(store !== 'walmart' || lowestNewOffer.seller === 'Walmart.com')
        {
          oldValue.offerHistory.push(lowestNewOffer)
          reason.push(CONSTANT.NEW_OFFER)
        }
      }
      return { store, value:  constructUpdateValue(newValue, hotItem, lowestNewOffer, oldValue.offerHistory) , reason: testPriceChange(reason, lowestNewOffer, oldValue)}
    } else if (oldValue.offer.available && newAvailOff.length === 0) { //means item went from avail to NOT avail
      const [lowestNonNewOffer] = newValue.offers.sort((x, y) => x.price - y.price)//find lowest of non avail offer
      //oldValue.offerHistory.push(lowestNonNewOffer)
      let reason = [CONSTANT.NOT_AVAILABLE, CONSTANT.AVAILABILITY_CHANGE]
      return { store, value:  constructUpdateValue(newValue, hotItem,lowestNonNewOffer, oldValue.offerHistory) , reason: testPriceChange(reason, lowestNonNewOffer, oldValue)}
    } else if(newAvailOff.length === 0) { //availablility didnt change but still not available, find lowest NON avail price
      const [lowestNonNewOffer] = newValue.offers.sort((x, y) => x.price - y.price)
      //oldValue.offerHistory.push(lowestNonNewOffer)
      let reason = [CONSTANT.NOT_AVAILABLE]
      return { store, value:  constructUpdateValue(newValue, hotItem,lowestNonNewOffer, oldValue.offerHistory) , reason: testPriceChange(reason, lowestNonNewOffer, oldValue)}
    } else { //availablility didnt change but still available, find lowest avail price
      let reason = [CONSTANT.AVAILABLE]
      // only add to offerHistory if the price changes and item is available
      if(oldValue.offer.price !== lowestNewOffer.price){
        if(store !== 'walmart' || lowestNewOffer.seller === 'Walmart.com')
        {
          oldValue.offerHistory.push(lowestNewOffer)
          reason.push(CONSTANT.NEW_OFFER)
        }
      }
      return { store, value:  constructUpdateValue(newValue, hotItem,lowestNewOffer, oldValue.offerHistory) , reason: testPriceChange(reason, lowestNewOffer, oldValue).concat(testMinorChange(oldValue, newValue))}
    }
  }
}
